@@ -29,6 +29,47 @@
 
 #include <string.h>
 
+#if defined(__GNUC__) && (__GNUC__ >= 4)
+# define _DBUS_ASSERT_ALIGNMENT(type, op, val) \
+  _DBUS_STATIC_ASSERT (__extension__ __alignof__ (type) op val)
+#else
+  /* not gcc, so probably no alignof operator: just use a no-op statement
+   * that's valid in the same contexts */
+# define _DBUS_ASSERT_ALIGNMENT(type, op, val) \
+  _DBUS_STATIC_ASSERT (TRUE)
+#endif
+
+/* True by definition, but just for completeness... */
+_DBUS_STATIC_ASSERT (sizeof (char) == 1);
+_DBUS_ASSERT_ALIGNMENT (char, ==, 1);
+
+_DBUS_STATIC_ASSERT (sizeof (dbus_int16_t) == 2);
+_DBUS_ASSERT_ALIGNMENT (dbus_int16_t, <=, 2);
+_DBUS_STATIC_ASSERT (sizeof (dbus_uint16_t) == 2);
+_DBUS_ASSERT_ALIGNMENT (dbus_uint16_t, <=, 2);
+
+_DBUS_STATIC_ASSERT (sizeof (dbus_int32_t) == 4);
+_DBUS_ASSERT_ALIGNMENT (dbus_int32_t, <=, 4);
+_DBUS_STATIC_ASSERT (sizeof (dbus_uint32_t) == 4);
+_DBUS_ASSERT_ALIGNMENT (dbus_uint32_t, <=, 4);
+_DBUS_STATIC_ASSERT (sizeof (dbus_bool_t) == 4);
+_DBUS_ASSERT_ALIGNMENT (dbus_bool_t, <=, 4);
+
+_DBUS_STATIC_ASSERT (sizeof (double) == 8);
+_DBUS_ASSERT_ALIGNMENT (double, <=, 8);
+
+_DBUS_STATIC_ASSERT (sizeof (dbus_int64_t) == 8);
+_DBUS_ASSERT_ALIGNMENT (dbus_int64_t, <=, 8);
+_DBUS_STATIC_ASSERT (sizeof (dbus_uint64_t) == 8);
+_DBUS_ASSERT_ALIGNMENT (dbus_uint64_t, <=, 8);
+
+_DBUS_STATIC_ASSERT (sizeof (DBusBasicValue) >= 8);
+/* The alignment of a DBusBasicValue might conceivably be > 8 because of the
+ * pointer, so we don't assert about it */
+
+_DBUS_STATIC_ASSERT (sizeof (DBus8ByteStruct) == 8);
+_DBUS_ASSERT_ALIGNMENT (DBus8ByteStruct, <=, 8);
+
 /**
  * @defgroup DBusMarshal marshaling and unmarshaling
  * @ingroup  DBusInternals
@@ -77,15 +118,10 @@ pack_8_octets (DBusBasicValue     value,
 {
   _dbus_assert (_DBUS_ALIGN_ADDRESS (data, 8) == data);
 
-#ifdef DBUS_HAVE_INT64
   if ((byte_order) == DBUS_LITTLE_ENDIAN)
     *((dbus_uint64_t*)(data)) = DBUS_UINT64_TO_LE (value.u64);
   else
     *((dbus_uint64_t*)(data)) = DBUS_UINT64_TO_BE (value.u64);
-#else
-  *(DBus8ByteStruct*)data = value.u64;
-  swap_8_octets ((DBusBasicValue*)data, byte_order);
-#endif
 }
 
 /**
@@ -103,65 +139,16 @@ _dbus_pack_uint32 (dbus_uint32_t   value,
   pack_4_octets (value, byte_order, data);
 }
 
-#ifndef DBUS_HAVE_INT64
-/* from ORBit */
-static void
-swap_bytes (unsigned char *data,
-            unsigned int   len)
-{
-  unsigned char *p1 = data;
-  unsigned char *p2 = data + len - 1;
-
-  while (p1 < p2)
-    {
-      unsigned char tmp = *p1;
-      *p1 = *p2;
-      *p2 = tmp;
-
-      --p2;
-      ++p1;
-    }
-}
-#endif /* !DBUS_HAVE_INT64 */
-
 static void
 swap_8_octets (DBusBasicValue    *value,
                int                byte_order)
 {
   if (byte_order != DBUS_COMPILER_BYTE_ORDER)
     {
-#ifdef DBUS_HAVE_INT64
       value->u64 = DBUS_UINT64_SWAP_LE_BE (value->u64);
-#else
-      swap_bytes ((unsigned char *)value, 8);
-#endif
     }
 }
 
-#if 0
-static DBusBasicValue
-unpack_8_octets (int                  byte_order,
-                 const unsigned char *data)
-{
-  DBusBasicValue r;
-
-  _dbus_assert (_DBUS_ALIGN_ADDRESS (data, 8) == data);
-  _dbus_assert (sizeof (r) == 8);
-
-#ifdef DBUS_HAVE_INT64
-  if (byte_order == DBUS_LITTLE_ENDIAN)
-    r.u64 = DBUS_UINT64_FROM_LE (*(dbus_uint64_t*)data);
-  else
-    r.u64 = DBUS_UINT64_FROM_BE (*(dbus_uint64_t*)data);
-#else
-  r.u64 = *(DBus8ByteStruct*)data;
-  swap_8_octets (&r, byte_order);
-#endif
-
-  return r;
-}
-#endif
-
 #ifndef _dbus_unpack_uint16
 /**
  * Unpacks a 16 bit unsigned integer from a data pointer
@@ -558,15 +545,10 @@ _dbus_marshal_read_basic (const DBusString      *str,
       {
       volatile dbus_uint64_t *vp = value;
       pos = _DBUS_ALIGN_VALUE (pos, 8);
-#ifdef DBUS_HAVE_INT64
       if (byte_order != DBUS_COMPILER_BYTE_ORDER)
         *vp = DBUS_UINT64_SWAP_LE_BE (*(dbus_uint64_t*)(str_data + pos));
       else
         *vp = *(dbus_uint64_t*)(str_data + pos);
-#else
-      *vp = *(DBus8ByteStruct*) (str_data + pos);
-      swap_8_octets (vp, byte_order);
-#endif
       pos += 8;
       }
       break;
@@ -617,7 +599,7 @@ marshal_2_octets (DBusString   *str,
   dbus_bool_t retval;
   int orig_len;
 
-  _dbus_assert (sizeof (value) == 2);
+  _DBUS_STATIC_ASSERT (sizeof (value) == 2);
 
   if (byte_order != DBUS_COMPILER_BYTE_ORDER)
     value = DBUS_UINT16_SWAP_LE_BE (value);
@@ -646,7 +628,7 @@ marshal_4_octets (DBusString   *str,
   dbus_bool_t retval;
   int orig_len;
 
-  _dbus_assert (sizeof (value) == 4);
+  _DBUS_STATIC_ASSERT (sizeof (value) == 4);
 
   if (byte_order != DBUS_COMPILER_BYTE_ORDER)
     value = DBUS_UINT32_SWAP_LE_BE (value);
@@ -675,7 +657,7 @@ marshal_8_octets (DBusString    *str,
   dbus_bool_t retval;
   int orig_len;
 
-  _dbus_assert (sizeof (value) == 8);
+  _DBUS_STATIC_ASSERT (sizeof (value) == 8);
 
   swap_8_octets (&value, byte_order);
 
@@ -922,11 +904,7 @@ _dbus_swap_array (unsigned char *data,
     {
       while (d != end)
         {
-#ifdef DBUS_HAVE_INT64
           *((dbus_uint64_t*)d) = DBUS_UINT64_SWAP_LE_BE (*((dbus_uint64_t*)d));
-#else
-          swap_8_bytes ((DBusBasicValue*) d);
-#endif
           d += 8;
         }
     }
@@ -1065,8 +1043,6 @@ _dbus_marshal_write_fixed_multi (DBusString *str,
     case DBUS_TYPE_INT16:
     case DBUS_TYPE_UINT16:
       return marshal_fixed_multi (str, insert_at, vp, n_elements, byte_order, 2, pos_after);
-      /* FIXME: we canonicalize to 0 or 1 for the single boolean case
-       * should we here too ? */
     case DBUS_TYPE_BOOLEAN:
     case DBUS_TYPE_INT32:
     case DBUS_TYPE_UINT32:
@@ -1233,44 +1209,6 @@ _dbus_type_get_alignment (int typecode)
     }
 }
 
-
-/**
- * Return #TRUE if the typecode is a valid typecode.
- * #DBUS_TYPE_INVALID surprisingly enough is not considered valid, and
- * random unknown bytes aren't either. This function is safe with
- * untrusted data.
- *
- * @returns #TRUE if valid
- */
-dbus_bool_t
-_dbus_type_is_valid (int typecode)
-{
-  switch (typecode)
-    {
-    case DBUS_TYPE_BYTE:
-    case DBUS_TYPE_BOOLEAN:
-    case DBUS_TYPE_INT16:
-    case DBUS_TYPE_UINT16:
-    case DBUS_TYPE_INT32:
-    case DBUS_TYPE_UINT32:
-    case DBUS_TYPE_INT64:
-    case DBUS_TYPE_UINT64:
-    case DBUS_TYPE_DOUBLE:
-    case DBUS_TYPE_STRING:
-    case DBUS_TYPE_OBJECT_PATH:
-    case DBUS_TYPE_SIGNATURE:
-    case DBUS_TYPE_ARRAY:
-    case DBUS_TYPE_STRUCT:
-    case DBUS_TYPE_DICT_ENTRY:
-    case DBUS_TYPE_VARIANT:
-    case DBUS_TYPE_UNIX_FD:
-      return TRUE;
-
-    default:
-      return FALSE;
-    }
-}
-
 /**
  * Returns a string describing the given type.
  *
@@ -1498,7 +1436,7 @@ _dbus_first_type_in_signature_c_str (const char       *str,
 
 /** @} */
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 #include "dbus-test.h"
 #include <stdio.h>
 
@@ -1642,7 +1580,7 @@ swap_test_array (void *array,
 #define DEMARSHAL_FIXED_ARRAY_AND_CHECK(typename, byte_order, literal)                  \
   do {                                                                                  \
     DEMARSHAL_FIXED_ARRAY (typename, byte_order);                                       \
-    if (memcmp (literal, v_ARRAY_##typename, sizeof (literal) != 0))                    \
+    if (memcmp (literal, v_ARRAY_##typename, sizeof (literal)) != 0)                    \
       {                                                                                 \
         _dbus_verbose ("MARSHALED DATA\n");                                             \
         _dbus_verbose_bytes_of_string (&str, dump_pos,                                  \
@@ -1671,12 +1609,10 @@ _dbus_marshal_test (void)
   unsigned char array1[5] = { 3, 4, 0, 1, 9 };
   dbus_int16_t array2[3] = { 124, 457, 780 };
   dbus_int32_t array4[3] = { 123, 456, 789 };
-#ifdef DBUS_HAVE_INT64
   dbus_int64_t array8[3] = { DBUS_INT64_CONSTANT (0x123ffffffff),
                              DBUS_INT64_CONSTANT (0x456ffffffff),
                              DBUS_INT64_CONSTANT (0x789ffffffff) };
   dbus_int64_t *v_ARRAY_INT64;
-#endif
   unsigned char *v_ARRAY_BYTE;
   dbus_int16_t *v_ARRAY_INT16;
   dbus_uint16_t *v_ARRAY_UINT16;
@@ -1732,7 +1668,6 @@ _dbus_marshal_test (void)
   MARSHAL_TEST (UINT32, DBUS_BIG_ENDIAN, 0x12345678);
   MARSHAL_TEST (UINT32, DBUS_LITTLE_ENDIAN, 0x12345678);
 
-#ifdef DBUS_HAVE_INT64
   /* Marshal signed integers */
   MARSHAL_TEST (INT64, DBUS_BIG_ENDIAN, DBUS_INT64_CONSTANT (-0x123456789abc7));
   MARSHAL_TEST (INT64, DBUS_LITTLE_ENDIAN, DBUS_INT64_CONSTANT (-0x123456789abc7));
@@ -1740,7 +1675,6 @@ _dbus_marshal_test (void)
   /* Marshal unsigned integers */
   MARSHAL_TEST (UINT64, DBUS_BIG_ENDIAN, DBUS_UINT64_CONSTANT (0x123456789abc7));
   MARSHAL_TEST (UINT64, DBUS_LITTLE_ENDIAN, DBUS_UINT64_CONSTANT (0x123456789abc7));
-#endif /* DBUS_HAVE_INT64 */
 
   /* Marshal byte */
   MARSHAL_TEST (BYTE, DBUS_BIG_ENDIAN, 5);
@@ -1782,10 +1716,8 @@ _dbus_marshal_test (void)
   MARSHAL_TEST_FIXED_ARRAY (BYTE, DBUS_BIG_ENDIAN, array1);
   MARSHAL_TEST_FIXED_ARRAY (BYTE, DBUS_LITTLE_ENDIAN, array1);
   
-#ifdef DBUS_HAVE_INT64
   MARSHAL_TEST_FIXED_ARRAY (INT64, DBUS_BIG_ENDIAN, array8);
   MARSHAL_TEST_FIXED_ARRAY (INT64, DBUS_LITTLE_ENDIAN, array8);
-#endif
 
 #if 0
 
@@ -1793,7 +1725,6 @@ _dbus_marshal_test (void)
    * FIXME restore the set/pack tests
    */
 
-#ifdef DBUS_HAVE_INT64
   /* set/pack 64-bit integers */
   _dbus_string_set_length (&str, 8);
 
@@ -1864,7 +1795,6 @@ _dbus_marshal_test (void)
   _dbus_assert (DBUS_UINT64_CONSTANT (0x123456789abc7) ==
                 _dbus_unpack_uint64 (DBUS_BIG_ENDIAN,
                                      _dbus_string_get_const_data (&str)));
-#endif /* DBUS_HAVE_INT64 */
 
   /* set/pack 32-bit integers */
   _dbus_string_set_length (&str, 4);
@@ -1989,4 +1919,4 @@ _dbus_marshal_test (void)
   return TRUE;
 }
 
-#endif /* DBUS_BUILD_TESTS */
+#endif /* DBUS_ENABLE_EMBEDDED_TESTS */
